#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-

from .agent_based_api.v1 import Service,Metric,Result,State,register,render

import re

def parse_http_log_parser(string_table):
   
    block = None
    result = {}
    for line in string_table:
        line_complete = " ".join(line)

        if line_complete.startswith("[") and line_complete.endswith("]"):
            block = line_complete[1:-1]
            result[block] = []
        else:
            result[block].append(line)

        if line_complete.startswith("error:"):
            #result[block] = line_complete
            result[block] = "error: {}".format(line_complete)
    
    return result


def discover_http_log_parser(section):
    for item in section.keys():
        yield Service(item=item)


def check_http_log_parser(item, params, section):
    base_regex= {
                 '101': '101',
                 '1XX': '1(?:0[023456789]|[1-9]\d)',
                 '200': '200',
                 '2XX': '2(?:0[1-9]|[1-9]0)',
                 '301': '301',
                 '302': '302',
                 '304': '304',
                 '3XX': '3[0-9]{2}',
                 '400': '400',
                 '401': '401',
                 '403': '403',
                 '404': '404',
                 '499': '499',
                 '4XX': '4(?:0[256789]|[1-8]\d)',
                 '500': '500',
                 '502': '502',
                 '503': '503',
                 '504': '504',
                 '5XX': '5(?:0[156789]|[1-9]\d)',
                 'XXX': '[05-9]\d\d',
                 'no_status': '\d*\D+\d*',
                }
    
    good_requests = ('101', '1XX', '200', '2XX', '301', '302', '304', '3XX')
    counter = {}
    request_sum = 0
    failure_rate = 0.0

    severity = State.OK
  
    perfdata = []

    for key in base_regex.keys():
        counter[key] = 0

    if not item in section:
        yield Result(state=State.UNKNOWN, summary="file not found by agent")
        return
    
    if section[item] and isinstance(section[item], str):
        raise Exception("error in agent plugin - " + section[item])
    
    for line in section[item]:
        (code, requests) = line

        for key, value in base_regex.items():
            if re.match(value, code):
                counter[key] += int(requests)
                
        request_sum += int(requests)

    
    if request_sum > 0:
        failure_rate = 100.0 - ((float(sum([counter[x] for x in good_requests])/float(request_sum)))*100.0)
        #text = "%d request%s, failure rate %s" % (request_sum, "s" if request_sum != 1 else "", render.percent(failure_rate))
        text = "{} request{}, failure rate {}".format(request_sum, "s" if request_sum != 1 else "", render.percent(failure_rate))
    else:
        text = "no new request found"
        
    if params:
        if "failure_rate" in params:
            (warn, crit) = params["failure_rate"]

            if failure_rate > crit:
                severity = State.CRIT
                text += " (over {:.2f}%)".format(crit)
            elif failure_rate > warn:
                severity = State.WARN
                text += " (over {:.2f}%)".format(warn)

    # Vérification des seuils pour count_1XX
    if params and "count_1XX" in params:
        (warn, crit) = params["count_1XX"]
        if counter["1XX"] > crit:
            yield Result(state=State.CRIT, summary="Count of 1XX responses is higher than {:.2f}".format(crit))
        elif counter["1XX"] > warn:
            yield Result(state=State.WARN, summary="Count of 1XX responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_2XX
    if params and "count_2XX" in params:
        (warn, crit) = params["count_2XX"]
        if counter["2XX"] > crit:
            yield Result(state=State.CRIT, summary="Count of 2XX responses is higher than {:.2f}".format(crit))
        elif counter["2XX"] > warn:
            yield Result(state=State.WARN, summary="Count of 2XX responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_3XX
    if params and "count_3XX" in params:
        (warn, crit) = params["count_3XX"]
        if counter["3XX"] > crit:
            yield Result(state=State.CRIT, summary="Count of 3XX responses is higher than {:.2f}".format(crit))
        elif counter["3XX"] > warn:
            yield Result(state=State.WARN, summary="Count of 3XX responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_401
    if params and "count_401" in params:
        (warn, crit) = params["count_401"]
        if counter["401"] > crit:
            yield Result(state=State.CRIT, summary="Count of 401 responses is higher than {:.2f}".format(crit))
        elif counter["401"] > warn:
            yield Result(state=State.WARN, summary="Count of 401 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_403
    if params and "count_403" in params:
        (warn, crit) = params["count_403"]
        if counter["403"] > crit:
            yield Result(state=State.CRIT, summary="Count of 403 responses is higher than {:.2f}".format(crit))
        elif counter["403"] > warn:
            yield Result(state=State.WARN, summary="Count of 403 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_404
    if params and "count_404" in params:
        (warn, crit) = params["count_404"]
        if counter["404"] > crit:
            yield Result(state=State.CRIT, summary="Count of 404 responses is higher than {:.2f}".format(crit))
        elif counter["404"] > warn:
            yield Result(state=State.WARN, summary="Count of 404 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_499
    if params and "count_499" in params:
        (warn, crit) = params["count_499"]
        if counter["499"] > crit:
            yield Result(state=State.CRIT, summary="Count of 499 responses is higher than {:.2f}".format(crit))
        elif counter["499"] > warn:
            yield Result(state=State.WARN, summary="Count of 499 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_4XX
    if params and "count_4XX" in params:
        (warn, crit) = params["count_4XX"]
        if counter["4XX"] > crit:
            yield Result(state=State.CRIT, summary="Count of 4XX responses is higher than {:.2f}".format(crit))
        elif counter["4XX"] > warn:
            yield Result(state=State.WARN, summary="Count of 4XX responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_500
    if params and "count_500" in params:
        (warn, crit) = params["count_500"]
        if counter["500"] > crit:
            yield Result(state=State.CRIT, summary="Count of 500 responses is higher than {:.2f}".format(crit))
        elif counter["500"] > warn:
            yield Result(state=State.WARN, summary="Count of 500 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_501
    if params and "count_501" in params:
        (warn, crit) = params["count_501"]
        if counter["501"] > crit:
            yield Result(state=State.CRIT, summary="Count of 501 responses is higher than {:.2f}".format(crit))
        elif counter["501"] > warn:
            yield Result(state=State.WARN, summary="Count of 501 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_502
    if params and "count_502" in params:
        (warn, crit) = params["count_502"]
        if counter["502"] > crit:
            yield Result(state=State.CRIT, summary="Count of 502 responses is higher than {:.2f}".format(crit))
        elif counter["502"] > warn:
            yield Result(state=State.WARN, summary="Count of 502 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_503
    if params and "count_503" in params:
        (warn, crit) = params["count_503"]
        if counter["503"] > crit:
            yield Result(state=State.CRIT, summary="Count of 503 responses is higher than {:.2f}".format(crit))
        elif counter["503"] > warn:
            yield Result(state=State.WARN, summary="Count of 503 responses is higher than {:.2f}".format(warn))

    # Vérification des seuils pour count_5XX
    if params and "count_5XX" in params:
        (warn, crit) = params["count_5XX"]
        if counter["5XX"] > crit:
            yield Result(state=State.CRIT, summary="Count of 5XX responses is higher than {:.2f}".format(crit))
        elif counter["5XX"] > warn:
            yield Result(state=State.WARN, summary="Count of 5XX responses is higher than {:.2f}".format(warn))
    # Ajoutez des vérifications similaires pour count_3XX, count_4XX et count_5XX

    yield Result(state=severity, summary=text)             
                    

    for key in counter.keys():
        yield Metric(name=("count_%s" % key), value=counter[key])

    yield Metric(name="failure_rate", value=failure_rate) 
    

register.agent_section(
    name="http_log_parser",
    parse_function = parse_http_log_parser,
)

register.check_plugin(
    name="http_log_parser",
    check_function = check_http_log_parser,
    discovery_function = discover_http_log_parser,
    service_name = "HTTP Log Parser %s",
    check_ruleset_name = "http_log_parser",
    check_default_parameters = {}
)
